/* CCS compiler-sepcific definitions for PIC16F877 */

#ID CHECKSUM                                    // ID bytes are checksum
#ZERO_RAM                                       // Wipe RAM for safety

#define WORD unsigned long  // Data type definitions
#define BOOL short          // 'short' is very short (1 bit) in this compiler

#define COMPILERID  "c"     // String to be displayed after signon

#ifdef __PCH__
#include <18C452.H>             // PIC18C452
#define _PIC18
#FUSES NOPROTECT,BROWNOUT       // Config fuse settings
#define PORTA_ADDR 0xf80        // Main I/O port addrs for PIC18
#define PORTB_ADDR 0xf81
#define PORTC_ADDR 0xf82
#define PORTD_ADDR 0xf83
#define PORTE_ADDR 0xf84
#byte TIMER_1_HIGH=0xfcf
#else
#include <16F877.h>             // PIC16F877
#device *=16
#FUSES HS,NOWDT,NOPROTECT,PUT,NOLVP    // Config fuse settings
#define PORTA_ADDR 5            // Main I/O port addrs for PIC16
#define PORTB_ADDR 6
#define PORTC_ADDR 7
#define PORTD_ADDR 8
#define PORTE_ADDR 9
#endif

#byte   PORTA = PORTA_ADDR      // I/O port definitions
#byte   PORTB = PORTB_ADDR
#byte   PORTC = PORTC_ADDR
#byte   PORTD = PORTD_ADDR
#byte   PORTE = PORTE_ADDR

#define SEPARATED #separate
#define BANK1
#define BANK2
#define BANK3
#define FAR

#define ALL_OUT 0
#define ALL_IN  0xff

// Bit I/O definition macros; rather strange, for CCS compatibility!
#define DEFBIT_7(reg, name) #BIT name = reg.7
#define DEFBIT_6(reg, name) #BIT name = reg.6
#define DEFBIT_5(reg, name) #BIT name = reg.5
#define DEFBIT_4(reg, name) #BIT name = reg.4
#define DEFBIT_3(reg, name) #BIT name = reg.3
#define DEFBIT_2(reg, name) #BIT name = reg.2
#define DEFBIT_1(reg, name) #BIT name = reg.1
#define DEFBIT_0(reg, name) #BIT name = reg.0
#define DEFBYTE(reg, name) #BYTE name = reg

#define HASHBYTE #BYTE
#use fast_io(A)                                 // I'll set the direction bits
#use fast_io(B)
#use fast_io(C)
#use fast_io(D)
#use fast_io(E)

#define DELAY_ONE_CYCLE delay_cycles(1)

#use DELAY(CLOCK=CPU_CLK)
#use RS232 (BAUD=SER_BAUD, XMIT=PIN_C6, RCV=PIN_C7, ERRORS)

#define init_serial()
#define init_i2c()

#define serial_putch(c) putchar(c)
#define putstr(str) printf(putch, str)
#define put_ser(str) printf(str)

#use I2C(MASTER, SDA=PIN_C4, SCL=PIN_C3, RESTART_WDT, FAST)

/* EOF */

