/* UDP functions for PIC Web server
** Copyright (c) Iosoft Ltd 2001
** This software is only licensed for distribution in the Iosoft ChipWeb package
** and may only be used for personal experimentation by the purchaser
** of that package, on condition that this copyright notice is retained.
** For commercial licensing, contact license@iosoft.co.uk
**
** This is experimental software; use it entirely at your own risk.
** Iosoft Ltd. offer no warranties with regard to this software */

#define UDPHDR_LEN  8       // UDP header length

WORD udplen;                // UDP length (data + header)
WORD ucsum;                 // UDP checksum

void udp_handler(void);
void put_udp(void);

/* Receive an incoming UDP datagram, return 0 if invalid */
SEPARATED BOOL udp_recv(void)
{
    WORD addr;

    checkhi = checklo = 0;
    checkflag = 0;
    DEBUG_PUTC('u');
    if (get_word(&remport) && get_word(&locport) && // Source & dest ports
        get_word(&udplen) && get_word(&ucsum))      // Dgram length, checksum
    {
        DEBUG_PUTC('>');
        rpdlen = udplen - UDPHDR_LEN;
        if (ucsum)
        {
            check_word(udplen);                     // Check pseudoheader
            check_lword(&myip);
            check_lword(&remip);
            check_byte(0);
            check_byte(PUDP);
            addr = getnic_addr();
            check_rxbytes(IPHDR_LEN+UDPHDR_LEN, rpdlen);
            setnic_addr(addr);
        }                                           // If checksum OK
        if (ucsum==0 || (checkhi==0xff && checklo==0xff))
        {                                           // ..call UDP handler
            udp_handler();
            return(1);
        }
    }
    DEBUG_PUTC('!');
    return(0);
}

/* Handle an incoming UDP datagram */
void udp_handler(void)
{
    tpdlen = tpxdlen = 0;               // Assume no Tx data
    checkhi = checklo = 0;              // Clear checksum
    checkflag = 0;
    DEBUG_PUTC('*');
    if (locport == ECHOPORT)            // Echo protocol: return copy of data
    {
        put_ip();                           // Put IP hdr in buffer
        put_word(locport);                  // Put UDP hdr in buffer
        put_word(remport);
        put_word(udplen);
        put_word(ucsum);                    // Re-use incoming checksum
        put_ether(txbuff, txin);            // Send UDP response
        copy_rx_tx(txin, IPHDR_LEN+UDPHDR_LEN, rpdlen);
        xmit_ether(IPHDR_LEN+UDPHDR_LEN+rpdlen);
        DEBUG_PUTC('U');
    }
    else if (locport == DAYPORT)       // Daytime protocol: return string
    {
        txin = IPHDR_LEN + UDPHDR_LEN;      // O/P data to buffer, calc checksum
        print_lcd = print_serial = print_nic = FALSE;
        print_net = TRUE;
        putstr(DAYMSG);                     // Send string to buffer
        d_checkhi = checkhi;                // Save checksum
        d_checklo = checklo;
        tpdlen = DAYMSG_LEN;                // Data length of response
        udplen = tpdlen + UDPHDR_LEN;       // UDP length
        iplen = udplen + IPHDR_LEN;         // IP length
        put_ip();                           // Put IP hdr in buffer
        put_udp();                          // Put UDP hdr in buffer
        put_ether(txbuff, txin + tpdlen);   // Copy hdrs and data into NIC
        xmit_ether(IPHDR_LEN + udplen);     // Transmit
        DEBUG_PUTC('U');
    }
}

/* Put out a UDP datagram. Data checksum must have already been computed */
void put_udp(void)
{
    checkflag = 0;                  // Ensure we're on an even byte
    checkhi = d_checkhi;            // Retrieve data checksum
    checklo = d_checklo;
    put_word(locport);              // Local and remote ports
    put_word(remport);
    put_word(udplen);               // UDP length
    check_lword(&myip);             // Add pseudoheader to checksum
    check_lword(&remip);
    check_byte(0);
    check_byte(PUDP);
    check_word(udplen);
    put_byte(~checkhi);             // Send checksum
    put_byte(~checklo);
}

/* EOF */
